package de.intevation.artifactdatabase.rest;

import org.apache.log4j.Logger;

import org.restlet.resource.ResourceException;

import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.data.MediaType;

import org.restlet.representation.Representation;
import org.restlet.representation.EmptyRepresentation;

import org.restlet.ext.xml.DomRepresentation;

import de.intevation.artifactdatabase.XMLUtils;

import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.ArtifactDatabaseException;
import de.intevation.artifacts.ArtifactDatabase;

import org.w3c.dom.Document;

import java.io.IOException;

/**
 * Resource to serve the out()-outputs of artifacts.
 * @author <a href="mailto:sascha.teichmann@intevation">Sascha L. Teichmann</a>
 */
public class ArtifactOutResource
extends      BaseResource
{
    /**
     * server URL where to find the resource.
     */
    public static final String PATH = "/artifact/{uuid}/{type}";

    /**
     * XPath to figure out the MIME type of the requested result.
     */
    public static final String XPATH_MIME_TYPE = "/art:action/art:out/art:mime-type/@value";

    /**
     * Default result MIME type: octet stream
     */
    public static final MediaType DEFAULT_MIME_TYPE =
        MediaType.APPLICATION_OCTET_STREAM;

    private static Logger logger = Logger.getLogger(ArtifactOutResource.class);

    @Override
    protected Representation innerPost(Representation requestRepr)
    throws    ResourceException
    {
        Document inputDocument = null;
        try {
            DomRepresentation input = new DomRepresentation(requestRepr);
            input.setNamespaceAware(true);
            inputDocument = input.getDocument();
        }
        catch (IOException ioe) {
            logger.error(ioe.getMessage());
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_BAD_REQUEST, ioe);
            return new EmptyRepresentation();
        }

        ArtifactDatabase db = (ArtifactDatabase)getContext()
            .getAttributes().get("database");

        Request request = getRequest();

        String identifier = (String)request.getAttributes().get("uuid");

        if (logger.isDebugEnabled()) {
            logger.debug("looking for artifact id '" + identifier + "'");
        }

        String mimeTypeString = XMLUtils.xpathString(
            inputDocument,
            XPATH_MIME_TYPE,
            ArtifactNamespaceContext.INSTANCE);

        MediaType mimeType = DEFAULT_MIME_TYPE;

        if (mimeTypeString != null && mimeTypeString.length() != 0) {
            try {
                mimeType = MediaType.valueOf(mimeTypeString);
            }
            catch (Exception e) {
                logger.error(e.getLocalizedMessage());
            }
        }

        try {
            return new OutRepresentation(
                mimeType,
                db.out(identifier, inputDocument, getCallMeta()));
        }
        catch (ArtifactDatabaseException adbe) {
            Response response = getResponse();
            response.setStatus(
                Status.CLIENT_ERROR_NOT_FOUND, adbe.getMessage());
            return new EmptyRepresentation();
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
